// $Id: ClipboardWatchDog.cs 25 2008-04-20 14:51:40Z arcao $
//
// Copyright (C) 2008 Martin "Arcao" Sloup <arcao@arcao.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace OSD.Program
{
    /// <summary>
    /// Trida na hlidani zmeny obsahu schranky
    /// </summary>
    public class ClipboardWatchDog : IDisposable
    {
        private readonly IntPtr windowHandle;
        private IntPtr nextClipboardViewer;

        /// <summary>
        /// Konstruktor tridy se zadanim handle okna, ktery zada o hlidani schranky
        /// </summary>
        /// <param name="windowHandle">handle okna</param>
        public ClipboardWatchDog(IntPtr windowHandle)
        {
            this.windowHandle = windowHandle;
            nextClipboardViewer = (IntPtr) SetClipboardViewer((int) windowHandle);
        }

        #region IDisposable Members

        /// <summary>
        /// Zrusi zaveseni detekce zmeny schranky
        /// </summary>
        public void Dispose()
        {
            ChangeClipboardChain(windowHandle, nextClipboardViewer);
        }

        #endregion

        [DllImport("User32.dll")]
        private static extern int SetClipboardViewer(int hWndNewViewer);

        [DllImport("User32.dll", CharSet = CharSet.Auto)]
        private static extern bool ChangeClipboardChain(IntPtr hWndRemove, IntPtr hWndNewNext);

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        private static extern int SendMessage(IntPtr hwnd, int wMsg, IntPtr wParam, IntPtr lParam);

        /// <summary>
        /// Udalost zmeny schranky
        /// </summary>
        public event EventHandler ClipboardChanged;

        /// <summary>
        /// Obsluhuje detekci schranky. Funkci je nutne volat ve WndProc formulare.
        /// </summary>
        /// <param name="m">zprava z WndProc</param>
        /// <returns>true, pokud je zprava odchycena a neni nutne ji posilat do bazove funkce</returns>
        public bool CheckWndProc(ref Message m)
        {
            // defined in winuser.h
            const int WM_DRAWCLIPBOARD = 0x308;
            const int WM_CHANGECBCHAIN = 0x030D;

            switch (m.Msg)
            {
                case WM_DRAWCLIPBOARD:
                    OnClipboardChanged();
                    SendMessage(nextClipboardViewer, m.Msg, m.WParam, m.LParam);
                    return true;

                case WM_CHANGECBCHAIN:
                    if (m.WParam == nextClipboardViewer)
                        nextClipboardViewer = m.LParam;
                    else
                        SendMessage(nextClipboardViewer, m.Msg, m.WParam, m.LParam);
                    return true;

                default:
                    return false;
            }
        }

        /// <summary>
        /// Zavola hadler oznamujici zmenu schranky
        /// </summary>
        protected virtual void OnClipboardChanged()
        {
            if (ClipboardChanged != null)
                ClipboardChanged(this, null);
        }
    }
}