﻿// $Id: DownloadPlugin.cs 25 2008-04-20 14:51:40Z arcao $
//
// Copyright (C) 2008 Martin "Arcao" Sloup <arcao@arcao.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text;
using System.Windows.Forms;
using OSD.PluginBase;

namespace OSD.Plugins.RapidshareComP
{
    ///<summary>
    /// Trida s DownloadPluginem pro Rapidshare.com Premium Account
    ///</summary>
    public class DownloadPlugin : PluginBase.DownloadPlugin
    {
        ///<summary>
        /// Konstruktor pluginu
        ///</summary>
        ///<param name="settings">trida s nastavenim</param>
        public DownloadPlugin(Settings settings) : base(settings) { }

        /// <summary>
        /// Vraci true, pokud DownloadPlugin implementujici urcity server ma premiove ucty 
        /// </summary>
        public override bool HasPremiumAccount
        {
            get { return true; }
        }

        ///<summary>
        /// Vraci nazev pluginu
        ///</summary>
        public override string Name
        {
            get { return "Rapidshare.com Premium"; }
        }

        ///<summary>
        /// Vraci autora pluginu
        ///</summary>
        public override string Author
        {
            get { return "Arcao"; }
        }

        ///<summary>
        /// Vraci e-mail autora pluginu
        ///</summary>
        public override string AuthorEMail
        {
            get { return "arcao@arcao.com"; }
        }

        ///<summary>
        /// Vraci url webu, kde se nachazi plugin, pripadne jeho aktualizace
        ///</summary>
        public override string PluginWeb
        {
            get { return "http://osd.arcao.com"; }
        }

        /// <summary>
        /// Zjisti, zda zadane url je schopne plugin stahnout
        /// </summary>
        /// <param name="url">url se souborem, ktery chceme stahnout</param>
        /// <returns>true, pokud plugin dokaze souboir na prislusnem url stahnout</returns>
        public override bool IsSuitableUrl(string url)
        {
            return Util.RegexMatch("(http://)?(www\\.)?rapidshare\\.com/files/\\d+/.+", url);
        }

        /// <summary>
        /// Vraci Download tridu, ktera se postara o stazeni souboru
        /// </summary>
        /// <param name="url">url se souborem, ktery se bude stahovat</param>
        /// <returns>Download tridu, ktera se postara o proces stahovani</returns>
        public override Download GetDownloadForUrl(string url)
        {
            if (IsSuitableUrl(url))
                return new RapidshareComPDownload(url, this);
            return null;
        }

        ///<summary>
        /// Obsahuje plugin dialog pro jeho podrobne nastaveni?
        ///</summary>
        public override bool HasSettingDialog
        {
            get { return true; }
        }

        ///<summary>
        /// Zobrazi dialog s podrobnym nastavenim pluginu. Plugin, ktery chce zobrzovat dialog s podrobnym nastavenim musi tuto metodu prepsat metodou zobrazujici dialogove okno s nastavenim.
        ///</summary>
        ///<param name="owner">vlastnik okna ()</param>
        ///<returns>navratovou hodnotu okna</returns>
        ///<exception cref="NotImplementedException">Pokud neni tato metoda prepsana, vraci NotImplementedException</exception>
        public override DialogResult ShowSettingDialog(IWin32Window owner)
        {
            string pluginPath = Settings.getPluginNodePath(PluginClassName);
            FormSettings f = new FormSettings();
            string user = Settings.getString(pluginPath + "/user", "-");

            f.Login = user.Substring(0, user.LastIndexOf('-'));
            f.Password = DecodePassword(user.Substring(user.LastIndexOf('-') + 1));

            DialogResult result = f.ShowDialog(owner);
            if (result == DialogResult.OK) {
                user = string.Format("{0}-{1}", f.Login, EncodePassword(f.Password));
                Settings.setString(pluginPath + "/user", user);
            }

            return result;
        }

        /// <summary>
        /// Zakoduje heslo dle Rapidshare.com zvyklosti
        /// </summary>
        /// <param name="password">heslo</param>
        /// <returns>zakodovane heslo</returns>
        public static string EncodePassword(string password)
        {
            if (password.Length == 0) return string.Empty;

            StringBuilder ret = new StringBuilder();
            byte[] bytes = ASCIIEncoding.ASCII.GetBytes(password);

            foreach (byte b in bytes)
                ret.Append("%" + b.ToString("X"));
            return ret.ToString();
        }

        /// <summary>
        /// Dekoduje heslo dle Rapidshare.com zvyklosti
        /// </summary>
        /// <param name="password">zakodovane heslo</param>
        /// <returns>dekodovane heslo</returns>
        public static string DecodePassword(string password)
        {
            if (password.Length == 0) return string.Empty;

            StringBuilder ret = new StringBuilder();
            string[] digits = password.Trim('%').Split('%');

            foreach (string digit in digits)
                ret.Append((char)Convert.ToInt32(digit, 16));

            return ret.ToString();
        }
    }
}