﻿// $Id: DownloadPlugin.cs 25 2008-04-20 14:51:40Z arcao $
//
// Copyright (C) 2008 Martin "Arcao" Sloup <arcao@arcao.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using OSD.PluginBase;

namespace OSD.Plugins.RapidshareCom
{
    ///<summary>
    /// Trida s DownloadPluginem pro Rapidshare.com
    ///</summary>
    public class DownloadPlugin : PluginBase.DownloadPlugin
    {
        ///<summary>
        /// Konstruktor pluginu
        ///</summary>
        ///<param name="settings">trida s nastavenim</param>
        public DownloadPlugin(Settings settings) : base(settings) {}

        /// <summary>
        /// Vraci true, pokud DownloadPlugin implementujici urcity server ma premiove ucty 
        /// </summary>
        public override bool HasPremiumAccount
        {
            get { return false; }
        }

        ///<summary>
        /// Vraci nazev pluginu
        ///</summary>
        public override string Name
        {
            get { return "Rapidshare.com"; }
        }

        ///<summary>
        /// Vraci autora pluginu
        ///</summary>
        public override string Author
        {
            get { return "Arcao"; }
        }

        ///<summary>
        /// Vraci e-mail autora pluginu
        ///</summary>
        public override string AuthorEMail
        {
            get { return "arcao@arcao.com"; }
        }

        ///<summary>
        /// Vraci url webu, kde se nachazi plugin, pripadne jeho aktualizace
        ///</summary>
        public override string PluginWeb
        {
            get { return "http://osd.arcao.com"; }
        }

        ///<summary>
        /// Obsahuje plugin dialog pro jeho podrobne nastaveni?
        ///</summary>
        public override bool HasSettingDialog
        {
            get { return true; }
        }

        /// <summary>
        /// Zjisti, zda zadane url je schopne plugin stahnout
        /// </summary>
        /// <param name="url">url se souborem, ktery chceme stahnout</param>
        /// <returns>true, pokud plugin dokaze souboir na prislusnem url stahnout</returns>
        public override bool IsSuitableUrl(string url)
        {
            return Util.RegexMatch("(http://)?(www\\.)?rapidshare\\.com/files/\\d+/.+", url);
        }

        /// <summary>
        /// Vraci Download tridu, ktera se postara o stazeni souboru
        /// </summary>
        /// <param name="url">url se souborem, ktery se bude stahovat</param>
        /// <returns>Download tridu, ktera se postara o proces stahovani</returns>
        public override Download GetDownloadForUrl(string url)
        {
            if (IsSuitableUrl(url))
                return new RapidshareComDownload(url, this);
            return null;
        }
    }
}