﻿// $Id: RapidshareComDownload.cs 26 2008-04-25 01:05:20Z arcao $
//
// Copyright (C) 2008 Martin "Arcao" Sloup <arcao@arcao.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Drawing;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;
using System.Threading;
using OSD.PluginBase;

namespace OSD.Plugins.RapidshareCom
{
    internal class RapidshareComDownload : Download
    {
        //private const int CONNECTION_TIMEOUT = 60000;
        private long position = 0;
        private long size = 0;

        private DownloadStatus status = DownloadStatus.Connecting;
        private string statusText = "";
        private int waitTime = 0;

        ///<summary>
        /// Konstruktor stahovaci tridy
        ///</summary>
        ///<param name="url">url souboru, ktery se bude stahovat</param>
        ///<param name="parrent">trida DownloadPluginu, kteremu nalezi tato trida</param>
        public RapidshareComDownload(string url, PluginBase.DownloadPlugin parrent) : base(url, parrent) {}

        ///<summary>
        /// Vraci stav stahovani
        ///</summary>
        public override DownloadStatus Status
        {
            get { return status; }
        }

        ///<summary>
        /// Vraci stav stahovani v textove podobe. Muze take obsahovat dalsi texty, ktery vratil server, odkud se stahuje. 
        ///</summary>
        public override string StatusText
        {
            get { return statusText; }
        }

        ///<summary>
        /// Vraci velikost stahovaneho souboru. Pokud jeste samotny proces stahovani nezacal vraci 0.
        ///</summary>
        public override long Size
        {
            get { return size; }
        }

        ///<summary>
        /// Vraci pocet bajtu, ktere jiz byli stahnuty
        ///</summary>
        public override long Position
        {
            get { return position; }
        }

        ///<summary>
        /// Vraci doporuceny cas, ktery je nutno pockat pro dalsi akci. Nastaveni tohoto gettru se provede pri Status == DownloadStatus.Waiting a Status == DownloadStatus.Timeout.
        ///</summary>
        public override int WaitTime
        {
            get { return waitTime; }
        }

        ///<summary>
        /// Funkce starajici se smazani pouzitych zdroju. Zavola se automaticky po zruseni tridy.
        ///</summary>
        protected override void ClearResources() {}

        ///<summary>
        /// Hlavni metoda starijici se o stahovani. Metoda je spustena v samostatnem vlakne. Zdedena trida musi tuto metodu implementovat.
        ///</summary>
        protected override void DownloadThread()
        {
            string content;
            GroupCollection group;

            status = DownloadStatus.Connecting;
            statusText = "Připojuji se...";
            OnStatusChanged(new DownloadStatusChangedEventArgs(status, statusText));

            try
            {
                wd.Clear();
                wd.Url = Url;
                wd.Method = MethodType.Get;
                wd.Send();
                content = wd.ResponseText;
            }
            catch (Exception ex)
            {
                status = DownloadStatus.Timeout;
                statusText = "Připojení se nezdařilo.";
                OnDownloadFailed(new DownloadFailedEventArgs(status, statusText, ex.Message));
                return;
            }

            if (content.Length == 0)
            {
                status = DownloadStatus.Timeout;
                statusText = "Odkaz nenalezen.";
                OnDownloadFailed(new DownloadFailedEventArgs(status, statusText, statusText));
                return;
            }

            if (
                !Util.RegexMatch(
                     "(?i)<form action=\"(?<url>http://[^.]+.rapidshare.com/files/\\d+/(?<filename>[^\"]+))\"[^>]+>",
                     content, out group))
            {
                //pokus o detekci smazaneho souboru
                //4 ... File not Found, 5 ... Deleted (by user/FORBIDDEN TO BE SHARED/...)
                if (content.IndexOf("<!-- E#4 -->") > -1 || content.IndexOf("<!-- E#5 -->") > -1)
                {
                    status = DownloadStatus.Deleted;
                    statusText = "Smazáno...";
                    OnDownloadFailed(new DownloadFailedEventArgs(status, statusText, statusText));
                    return;
                }

                status = DownloadStatus.Timeout;
                statusText = "Odkaz nenalezen.";
                OnDownloadFailed(new DownloadFailedEventArgs(status, statusText, statusText));
                return;
            }

            // nyni ziskame stranku se zadanim obrazku a vyberu odkud stahovat
            string newUrl = group["url"].Value;

            try
            {
                wd.Clear();
                wd.Url = newUrl;
                wd.Method = MethodType.Post;
                wd.QueryString.Add("dl.start", "Free");
                wd.Referer = Url;
                wd.Send();
                content = wd.ResponseText;
            }
            catch (Exception ex)
            {
                status = DownloadStatus.Timeout;
                statusText = "Připojení se nezdařilo.";
                OnDownloadFailed(new DownloadFailedEventArgs(status, statusText, ex.Message));
                return;
            }

            if (content.Length == 0)
            {
                status = DownloadStatus.Timeout;
                statusText = "Odkaz nenalezen.";
                OnDownloadFailed(new DownloadFailedEventArgs(status, statusText, statusText));
                return;
            }

            GroupCollection groupImg;

            //FIXME lepsi ziskani odkazu, moznost dle nastaveni pluginu urcit v jakem poradi vybrat vhodny server
            if (
                !Util.RegexMatch(
                     "(?i)action=\"(?<url>http://[^.]+.rapidshare.com/files/\\d+/\\d+/(?<filename>[^\"]+)(.html)?)\" method=\"post\">",
                     content, out group))
            {
                //TODO test na cekani pro dalsi stahovani
                status = DownloadStatus.Timeout;
                statusText = "Odkaz nenalezen.";
                OnDownloadFailed(new DownloadFailedEventArgs(status, statusText, statusText));
                return;
            }

            string downloadUrl = group["url"].Value;
            string captchaCode = "";

            string localFilename =
                Path.Combine(Parrent.Settings.getString("osd/settings/destination_path", @"c:\"),
                             group["filename"].Value);

            // pokud neni captcha muze se jednat o nejakou akci typu happy hour.
            if (Util.RegexMatch("'<img src=\"(?<url>[^\"]+)\"><br>here", content, out groupImg))
            {
                string imageUrl = groupImg["url"].Value;

                GroupCollection waitG;

                // zjistime, jak mame dluho cekat
                waitTime = 0;
                if (Util.RegexMatch("<script>var c=(?<wait>\\d+); var", content, out waitG))
                {
                    try
                    {
                        // projistotu pridame 4 sekundy
                        waitTime = int.Parse(waitG["wait"].Value) + 4;
                    }
                    catch {}
                }


                //stahneme captcha obrazek
                try
                {
                    wd.Clear();
                    wd.Url = imageUrl;
                    wd.Referer = newUrl;
                    wd.Method = MethodType.Get;
                    wd.Send();

                    if (waitTime > 0)
                        OnStatusChanged(new WaitingStatusChangeEventArgs(DownloadStatus.Waiting, "", waitTime));

                    DateTime timeStart = DateTime.Now;

                    //zobrazeni dialogu pro zadani obrazku
                    FormCaptcha captcha = new FormCaptcha(Image.FromStream(wd.ResponseStream, false));
                    captcha.ShowDialog();
                    captchaCode = captcha.Code;

                    TimeSpan t = (DateTime.Now - timeStart);

                    // pokud jeste necekame dostatecnou dobu, uspime vlakno na zbytek casu
                    if (t.TotalSeconds < waitTime) Thread.Sleep((int) (waitTime - t.TotalSeconds)*1000);
                    waitTime = 0;
                }
                catch
                {
                    status = DownloadStatus.CaptchaFailed;
                    statusText = "Chyba při získávání captcha obrázku.";
                    OnDownloadFailed(new DownloadFailedEventArgs(status, statusText, statusText));
                    return;
                }
            }

            try
            {
                wd.Clear();
                wd.Url = downloadUrl;
                wd.Referer = newUrl;
                wd.Method = MethodType.Post;
                wd.QueryString.Add("mirror", "on");
                wd.QueryString.Add("accesscode", captchaCode);
                //FIXME je nutne ziskat presny nazev tlacitka, takovy co nastavi javascript na strance
                wd.QueryString.Add("actionstring", "Download via");

                //pokud existuje, pokusime se nastavit, ze chceme pokracovat
                if (File.Exists(localFilename))
                    wd.AddRange = new FileInfo(localFilename).Length;

                wd.Send();
                //TODO test na captcha neni spravna
                if (wd.Response == null || wd.Response.ContentType.IndexOf("text/html") > -1)
                {
                    throw new WebException();
                }

                status = DownloadStatus.Downloading;
                statusText = "Stahuji...";
                OnStatusChanged(new DownloadStatusChangedEventArgs(status, statusText));

                wd.SaveToFile(localFilename, delegate(object sender, SaveToFileProgressChangedEventArgs e)
                                                 {
                                                     position = e.BytesReceived;
                                                     size = e.TotalBytesToReceive;
                                                     OnPositionChanged(new PositionChangedEventArgs(position, size));
                                                 });
                status = DownloadStatus.Downloaded;
                statusText = "";

                OnStatusChanged(new DownloadStatusChangedEventArgs(status, statusText));
                OnDownloadComplete(new DownloadCompleteEventArgs(Url, localFilename));
            }
            catch (Exception e)
            {
                status = DownloadStatus.IOError;
                statusText = "Stahování se nezdařilo.";
                OnDownloadFailed(new DownloadFailedEventArgs(status, statusText, e.Message));
                return;
            }
        }
    }
}