﻿// $Id: Util.cs 25 2008-04-20 14:51:40Z arcao $
//
// Copyright (C) 2008 Martin "Arcao" Sloup <arcao@arcao.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System.Drawing;
using System.Windows.Forms;

namespace OSD.Program
{
    /// <summary>
    /// Pomocna trida programu
    /// </summary>
    public static class Util
    {
        private const long GB = 1073741824;
        private const long KB = 1024;
        private const long MB = 1048576;
        private const long TB = 1099511627776;

        /// <summary>
        /// Vraci formatovany retezec pro zadane cislo jako [cislo] [velikost], kde cislo je velikost formatovana na jedno desetine cislo a velikost je jednotka velikosti: TB, GB, MB, KB nebo B.
        /// </summary>
        /// <param name="size">velikost</param>
        /// <returns>formatovany retezec</returns>
        public static string SizeFormat(long size)
        {
            float fSize = size;
            if (size > TB)
                return string.Format("{0:0.0} TB", fSize/TB);
            if (size > GB)
                return string.Format("{0:0.0} GB", fSize/GB);
            if (size > MB)
                return string.Format("{0:0.0} MB", fSize/MB);
            if (size > KB)
                return string.Format("{0:0.0} KB", fSize/KB);

            return string.Format("{0} B", size);
        }

        /// <summary>
        /// Nastavi vsem polozkam okna, zadane pres ControlCollection systemovy font.
        /// </summary>
        /// <param name="controls">polozky okna</param>
        public static void SetSystemFont(Control.ControlCollection controls)
        {
            foreach (Control c in controls)
            {
                if (Control.DefaultFont.Size == c.Font.Size)
                    c.Font = new Font(SystemFonts.DialogFont.FontFamily, SystemFonts.DialogFont.Size, c.Font.Style);
                else
                    c.Font = new Font(SystemFonts.DialogFont.FontFamily, c.Font.Size, c.Font.Style);
            }
        }
    }
}